@@ -1,6 +1,8 @@
 //! RakNet Protocol implementation by Rust.
 //! 
-//! Raknet is a reliable udp transport protocol that is often used for communication between game clients and servers. This project is an incomplete implementation of the protocol.
+//! Raknet is a reliable udp transport protocol that is generally used for communication between game clients and servers, and is used by Minecraft Bedrock Edtion for underlying communication.
+//! 
+//! Raknet protocol supports multiple reliability, and has better transmission performance than TCP in unstable network environments. This project is an incomplete implementation of the protocol.
 //! 
 //! Reference : <http://www.jenkinssoftware.com/raknet/manual/index.html>
 //! 
@@ -11,6 +13,7 @@
 //! * Async
 //! * MIT License
 //! * Pure Rust implementation
+//! * Fast Retransmission
 //! * Selective Retransmission (TCP/Full Retransmission)
 //! * Non-delayed ACK (TCP/Delayed ACK)
 //! * RTO Not Doubled (TCP/RTO Doubled)
@@ -22,7 +25,7 @@
 //! ```toml
 //! # Cargo.toml
 //! [dependencies]
-//! rust-raknet = "0.5.1"
+//! rust-raknet = "*"
 //! ```
 //! 
 //! # Reliability
@@ -372,6 +375,46 @@ async fn test_loss_packet_with_sequenced(){
     notify2.notified().await;
 }
 
+#[tokio::test]
+async fn test_async_read_write_trait(){
+    let mut server = RaknetListener::bind("127.0.0.1:0".parse().unwrap()).await.unwrap();
+    let local_addr = server.local_addr().unwrap();
+    server.listen().await;
+    tokio::spawn(async move {
+        let mut client1 = server.accept().await.unwrap();
+        assert!(client1.local_addr().unwrap() == local_addr);
+        tokio::io::AsyncWriteExt::write(&mut client1, &[1,2,3]).await.unwrap();
+        tokio::io::AsyncWriteExt::write(&mut client1, &[0xfe,4,5,6]).await.unwrap();
+        tokio::io::AsyncWriteExt::write(&mut client1, &[0xfe,7,8,9]).await.unwrap();
+        tokio::io::AsyncWriteExt::write(&mut client1, &[1,2,3]).await.unwrap();
+    });
+    let mut client2 = RaknetSocket::connect(&local_addr).await.unwrap();
+    assert!(client2.peer_addr().unwrap() == local_addr);
+    let mut buf : Vec<u8> = vec![0u8;1];
+    tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
+    assert!(buf == vec![1]);
+    tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
+    assert!(buf == vec![2]);
+    tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
+    assert!(buf == vec![3]);
+
+    let mut buf : Vec<u8> = vec![0u8;4];
+    tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
+    assert!(buf == vec![0xfe, 4,5,6]);
+
+    let mut buf : Vec<u8> = vec![0u8;1];
+    tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
+    assert!(buf == vec![0xfe]);
+
+    let mut buf : Vec<u8> = vec![0u8;3];
+    tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
+    assert!(buf == vec![7,8,9]);
+
+    let mut buf : Vec<u8> = vec![0u8;3];
+    tokio::io::AsyncReadExt::read(&mut client2, &mut buf).await.unwrap();
+    assert!(buf == vec![1,2,3]);
+}
+
 /*
 #[tokio::test]
 async fn chore(){
