@@ -1,12 +1,13 @@
 use std::{net::{SocketAddr}, sync::{Arc, atomic::{AtomicU8, AtomicI64}}};
+use futures::FutureExt;
 use rand::Rng;
-use tokio::{net::UdpSocket, sync::{Mutex, mpsc::channel, Notify}, time::{sleep, timeout}};
+use tokio::{net::UdpSocket, sync::{Mutex, mpsc::channel, Notify}, time::{sleep, timeout}, io::{AsyncRead, AsyncWrite}};
 
 use tokio::sync::mpsc::{Sender, Receiver};
 use std::sync::atomic::{AtomicBool, Ordering};
-use crate::{error::{Result, RaknetError}};
+use crate::{error::{Result, RaknetError}, raknet_log_error, raknet_log_info};
 
-use crate::{packet::*, utils::*, arq::*, raknet_log};
+use crate::{packet::*, utils::*, arq::*, raknet_log_debug};
 
 /// Raknet socket wrapper with local and remote.
 pub struct RaknetSocket{
@@ -21,6 +22,7 @@ pub struct RaknetSocket{
     enable_loss : Arc<AtomicBool>,
     loss_rate : Arc<AtomicU8>,
     incomming_notify : Arc<Notify>,
+    async_read_remaining : Vec<u8>
 }
 
 impl RaknetSocket {
@@ -43,6 +45,7 @@ impl RaknetSocket {
             enable_loss : Arc::new(AtomicBool::new(false)),
             loss_rate : Arc::new(AtomicU8::new(0)),
             incomming_notify : Arc::new(Notify::new()),
+            async_read_remaining : vec![]
         };
         ret.start_receiver(receiver , user_data_sender);
         ret.start_tick(Some(collecter));
@@ -85,7 +88,7 @@ impl RaknetSocket {
                 //i dont know why incomming packet after always follow a connected ping packet in minecraft bedrock 1.18.12.
                 let buf = write_packet_connected_ping(&ping).await?;
                 sendq.insert(Reliability::Unreliable ,&buf)?;
-                raknet_log!("incomming notified");
+                raknet_log_debug!("incomming notified");
                 incomming_notify.notify_one();
             }
             PacketID::NewIncomingConnection => {
@@ -123,11 +126,17 @@ impl RaknetSocket {
             let mut rng = rand::thread_rng();
             let i: u8 = rng.gen_range(0..11);
             if i > loss_rate.load(Ordering::Relaxed) {
-                raknet_log!("loss packet");
+                raknet_log_debug!("loss packet");
                 return Ok(0);
             }
         }
-        s.send_to(buf, target).await
+        match s.send_to(buf, target).await{
+            Ok(p) => Ok(p),
+            Err(e) => {
+                raknet_log_error!("udp socket send_to error : {}" ,e);
+                Ok(0)
+            },
+        }
     }
     
     /// Connect to a Raknet server and return a Raknet socket
@@ -165,11 +174,24 @@ impl RaknetSocket {
         let mut reply1_buf =  [0u8 ; 2048];
 
         loop{
-            s.send_to(&buf, addr).await.unwrap();
-            let (size ,src ) = match timeout(std::time::Duration::from_secs(2),s.recv_from(&mut reply1_buf)).await{
-                Ok(p) => p.unwrap(),
+            match s.send_to(&buf, addr).await{
+                Ok(p) => p,
+                Err(e) => {
+                    raknet_log_error!("udp socket sendto error {}" , e);
+                    tokio::time::sleep(std::time::Duration::from_secs(2)).await;
+                    continue;
+                },
+            };
+            let (size ,src ) = match match timeout(std::time::Duration::from_secs(2),s.recv_from(&mut reply1_buf)).await{
+                Ok(p) => p,
                 Err(_) =>{
-                    raknet_log!("wait reply1 timeout");
+                    raknet_log_debug!("wait reply1 timeout");
+                    continue;
+                }
+            }{
+                Ok(p) => p,
+                Err(e) => {
+                    raknet_log_error!("recvfrom error : {}" , e);
                     continue;
                 }
             };
@@ -186,7 +208,7 @@ impl RaknetSocket {
     
                     return Err(RaknetError::NotSupportVersion);
                 }else{
-                    raknet_log!("incorrect reply1");
+                    raknet_log_debug!("incorrect reply1");
                     return Err(RaknetError::IncorrectReply);
                 }
             }
@@ -210,19 +232,32 @@ impl RaknetSocket {
         let buf = write_packet_connection_open_request_2(&packet).await.unwrap();
 
         loop{
-            s.send_to(&buf, addr).await.unwrap();
-
-            let mut buf = [0u8 ; 2048];
-            let (size ,_ ) = match timeout(std::time::Duration::from_secs(2) ,s.recv_from(&mut buf)).await{
-                Ok(p) => p.unwrap(),
-                Err(_) => {
-                    raknet_log!("wait reply2 timeout");
+            match s.send_to(&buf, addr).await{
+                Ok(_) => {},
+                Err(e) => {
+                    raknet_log_error!("udp socket sendto error {}" , e);
+                    tokio::time::sleep(std::time::Duration::from_secs(2)).await;
                     continue;
                 },
             };
 
+            let mut buf = [0u8 ; 2048];
+            let (size ,_ ) = match match timeout(std::time::Duration::from_secs(2) ,s.recv_from(&mut buf)).await{
+                Ok(p) => p,
+                Err(_) => {
+                    raknet_log_debug!("wait reply2 timeout");
+                    continue;
+                },
+            }{
+                Ok(p) => p,
+                Err(e) => {
+                    raknet_log_error!("recvfrom error : {}" , e);
+                    continue;
+                }
+            };
+
             if buf[0] != PacketID::OpenConnectionReply2.to_u8(){
-                raknet_log!("incorrect reply2");
+                raknet_log_debug!("incorrect reply2");
                 return Err(RaknetError::IncorrectReply);
             }
     
@@ -273,10 +308,10 @@ impl RaknetSocket {
                         #[cfg(target_family = "windows")]
                         if e.raw_os_error().unwrap() == 10040{
                             // https://docs.microsoft.com/zh-CN/troubleshoot/windows-server/networking/wsaemsgsize-error-10040-in-winsock-2
-                            raknet_log!("recv_from error : {}" , e.raw_os_error().unwrap());
+                            raknet_log_debug!("recv_from error : {}" , e.raw_os_error().unwrap());
                             continue;
                         }
-                        raknet_log!("recv_from error : {}" , e);
+                        raknet_log_debug!("recv_from error : {}" , e);
                         connected_s.store(false, Ordering::Relaxed);
                         break;
                     },
@@ -285,13 +320,13 @@ impl RaknetSocket {
                 match sender.send(buf[..size].to_vec()).await{
                     Ok(_) => {},
                     Err(e) => {
-                        raknet_log!("channel send error : {}" , e);
+                        raknet_log_debug!("channel send error : {}" , e);
                         connected_s.store(false, Ordering::Relaxed);
                         break;
                     },
                 };
             }
-            raknet_log!("{} , recv_from finished" , peer_addr );
+            raknet_log_debug!("{} , recv_from finished" , peer_addr );
         });
 
         let ret = RaknetSocket{
@@ -306,12 +341,13 @@ impl RaknetSocket {
             enable_loss : Arc::new(AtomicBool::new(false)),
             loss_rate : Arc::new(AtomicU8::new(0)),
             incomming_notify : Arc::new(Notify::new()),
+            async_read_remaining : vec![]
         };
 
         ret.start_receiver(receiver , user_data_sender);
         ret.start_tick(None);
 
-        raknet_log!("wait incomming notify");
+        raknet_log_debug!("wait incomming notify");
         ret.incomming_notify.notified().await;
         
         Ok(ret)
@@ -337,7 +373,7 @@ impl RaknetSocket {
                 let buf = match receiver.recv().await{
                     Some(buf) => buf,
                     None => {
-                        raknet_log!("channel receiver finished");
+                        raknet_log_debug!("channel receiver finished");
                         connected.store(false, Ordering::Relaxed);
                         break;
                     },
@@ -398,7 +434,7 @@ impl RaknetSocket {
 
                         for f in recvq.flush(&peer_addr){
                             if !RaknetSocket::handle(&f , &peer_addr ,&local_addr, &sendq, &user_data_sender , &incomming_notify).await.unwrap(){
-                                raknet_log!("handle error");
+                                raknet_log_info!("handle over");
                                 connected.store(false, Ordering::Relaxed);
                                 is_break = true;
                                 break;
@@ -413,7 +449,7 @@ impl RaknetSocket {
                     //flush ack
                     let acks = recvq.get_ack();
 
-                    if acks.len() != 0 {
+                    if !acks.is_empty() {
     
                         let packet = ACK{
                             record_count: acks.len() as u16,
@@ -424,11 +460,11 @@ impl RaknetSocket {
                         RaknetSocket::sendto(&s , &buf, &peer_addr , &enable_loss  , &loss_rate).await.unwrap();
                     }
                 } else {
-                    raknet_log!("unknown packetid : {}", buf[0]);
+                    raknet_log_debug!("unknown packetid : {}", buf[0]);
                 }
             }
 
-            raknet_log!("{} , receiver finished" , peer_addr);
+            raknet_log_debug!("{} , receiver finished" , peer_addr);
         });
     }
 
@@ -449,19 +485,14 @@ impl RaknetSocket {
                 // flush nack
                 let mut recvq = recvq.lock().await;
                 let nacks = recvq.get_nack();
-                if nacks.len() != 0{
+                if !nacks.is_empty(){
                     let nack = NACK{
                         record_count: nacks.len() as u16,
                         sequences: nacks,
                     };
 
                     let buf = write_packet_nack(&nack).await.unwrap();
-                    match RaknetSocket::sendto(&s , &buf, &peer_addr , &enable_loss , &loss_rate).await{
-                        Ok(_) => {},
-                        Err(_) => {
-                            break;
-                        },
-                    };
+                    RaknetSocket::sendto(&s , &buf, &peer_addr , &enable_loss , &loss_rate).await.unwrap();
                 }
                 
                 //flush sendq
@@ -473,7 +504,7 @@ impl RaknetSocket {
 
                 //monitor log
                 if cur_timestamp_millis() - last_monitor_tick > 10000{
-                    raknet_log!("peer addr : {} , sendq size : {} , sentq size : {} , rto : {} , recvq size : {} ,  recvq fragment size : {} , ordered queue size : {} - {:?}" , 
+                    raknet_log_debug!("peer addr : {} , sendq size : {} , sentq size : {} , rto : {} , recvq size : {} ,  recvq fragment size : {} , ordered queue size : {} - {:?}" , 
                         peer_addr,
                         sendq.get_reliable_queue_size(),
                         sendq.get_sent_queue_size(),
@@ -488,7 +519,7 @@ impl RaknetSocket {
 
                 // if exceed 60s not received any packet will close connection.
                 if cur_timestamp_millis() - last_heartbeat_time.load(Ordering::Relaxed) > RECEIVE_TIMEOUT{
-                    raknet_log!("recv timeout");
+                    raknet_log_debug!("recv timeout");
                     connected.store(false, Ordering::Relaxed);
                     break;
                 }
@@ -505,7 +536,7 @@ impl RaknetSocket {
                 },
                 None => {},
             }
-            raknet_log!("{} , ticker finished" , peer_addr);
+            raknet_log_debug!("{} , ticker finished" , peer_addr);
         });
     }
 
@@ -549,7 +580,17 @@ impl RaknetSocket {
 
         let buf = write_packet_ping(&packet).await?;
 
-        s.send_to(buf.as_slice(), addr).await.unwrap();
+        loop{
+            match s.send_to(buf.as_slice(), addr).await{
+                Ok(_) => break,
+                Err(e) => {
+                    raknet_log_error!("udp socket sendto error {}" , e);
+                    tokio::time::sleep(std::time::Duration::from_secs(2)).await;
+                    continue;
+                },
+            };
+        }
+
 
         let mut buf = [0u8 ; 1024];
         match s.recv_from(&mut buf).await{
@@ -656,4 +697,88 @@ impl RaknetSocket {
         self.loss_rate.store(stage, Ordering::Relaxed);
 
     }
+}
+
+/// Note: AsyncRead requires you to provide a fixed-size buffer for reading data.
+/// 
+/// However, Raknet is not a streaming protocol, so when you use AsyncRead, if you don't provide enough buffer size (less than Raknet Packet size), then a Raknet Packet will be split into multiple reads.
+/// 
+/// This will lead to a similar TCP unpacking problem when you use AsyncRead when you are developing scenarios such as Raknet reverse proxy where the packet size cannot be predicted.
+/// https://programmer.ink/think/tcp-packet-sticking-unpacking.html
+/// So it is not recommended to use AsyncRead for developing Raknet reverse proxy.
+/// 
+impl AsyncRead for RaknetSocket {
+    fn poll_read(
+        mut self: std::pin::Pin<&mut Self>,
+        cx: &mut std::task::Context<'_>,
+        buf: &mut tokio::io::ReadBuf<'_>,
+    ) -> std::task::Poll<std::io::Result<()>> {
+
+        if !self.async_read_remaining.is_empty() {
+
+            if buf.remaining() >= self.async_read_remaining.len() {
+                buf.put_slice(&self.async_read_remaining);
+                self.async_read_remaining.clear();
+            }else {
+                let pos = buf.remaining();
+                buf.put_slice(&self.async_read_remaining[..pos]);
+                self.async_read_remaining = self.async_read_remaining[pos..].to_vec();
+            }
+            return std::task::Poll::Ready(Ok(()));
+        }
+
+        match self.user_data_receiver.poll_recv(cx){
+            std::task::Poll::Ready(data) => {
+
+                if data != None{
+                    let data = data.unwrap();
+
+                    if buf.remaining() >= data.len() {
+                        buf.put_slice(&data);
+                    }else {
+                        let pos = buf.remaining();
+                        buf.put_slice(&data[..pos]);
+                        self.async_read_remaining = data[pos..].to_vec();
+                    }
+
+                    std::task::Poll::Ready(Ok(()))
+                } else {
+                    std::task::Poll::Ready(Err(std::io::Error::new(std::io::ErrorKind::ConnectionReset, "connection reset")))
+                }
+            },
+            std::task::Poll::Pending => std::task::Poll::Pending,
+        }
+    }
+}
+
+/// Note: AsyncWrite is not stream written. 
+/// Every time the data sent using AsyncWrite will be completely put into the Raknet Packet, so the returned write size is always equal to the incoming buffer size.
+impl AsyncWrite for RaknetSocket {
+    fn poll_write(
+        self: std::pin::Pin<&mut Self>,
+        cx: &mut std::task::Context<'_>,
+        buf: &[u8],
+    ) -> std::task::Poll<std::io::Result<usize>> {
+
+        if !self.connected.load(Ordering::Relaxed) {
+            return std::task::Poll::Ready(Err(std::io::Error::new(std::io::ErrorKind::ConnectionReset, "connection reset")));
+        }
+
+        match self.sendq.lock().boxed_local().poll_unpin(cx){
+            std::task::Poll::Ready(mut p) => {
+                p.insert(Reliability::ReliableOrdered, buf).unwrap();
+                std::task::Poll::Ready(Ok(buf.len()))
+            },
+            std::task::Poll::Pending => std::task::Poll::Pending,
+        }
+
+    }
+
+    fn poll_flush(self: std::pin::Pin<&mut Self>, _cx: &mut std::task::Context<'_>) -> std::task::Poll<std::io::Result<()>> {
+        std::task::Poll::Ready(Ok(()))
+    }
+
+    fn poll_shutdown(self: std::pin::Pin<&mut Self>, _cx: &mut std::task::Context<'_>) -> std::task::Poll<std::io::Result<()>> {
+        std::task::Poll::Ready(Ok(()))
+    }
 }
\ No newline at end of file
